@@ -30,9 +30,16 @@ int32_t SPVM__Net__SSLeay__PEM__read_bio_X509(SPVM_ENV* env, SPVM_VALUE* stack)
     
     env->die(env, stack, "[OpenSSL Error]PEM_read_bio_X509 failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
     
-    int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
-    if (error_id) { return error_id; }
-    error_id = tmp_error_id;
+    if (ERR_GET_REASON(ssl_error) == PEM_R_NO_START_LINE) {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error::PEM_R_NO_START_LINE", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
+    else {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
     
     return error_id;
   }
@@ -71,9 +78,16 @@ int32_t SPVM__Net__SSLeay__PEM__read_bio_X509_CRL(SPVM_ENV* env, SPVM_VALUE* sta
     
     env->die(env, stack, "[OpenSSL Error]PEM_read_bio_X509_CRL failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
     
-    int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
-    if (error_id) { return error_id; }
-    error_id = tmp_error_id;
+    if (ERR_GET_REASON(ssl_error) == PEM_R_NO_START_LINE) {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error::PEM_R_NO_START_LINE", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
+    else {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
     
     return error_id;
   }
@@ -112,9 +126,16 @@ int32_t SPVM__Net__SSLeay__PEM__read_bio_DHparams(SPVM_ENV* env, SPVM_VALUE* sta
     
     env->die(env, stack, "[OpenSSL Error]PEM_read_bio_DHparams failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
     
-    int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
-    if (error_id) { return error_id; }
-    error_id = tmp_error_id;
+    if (ERR_GET_REASON(ssl_error) == PEM_R_NO_START_LINE) {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error::PEM_R_NO_START_LINE", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
+    else {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
     
     return error_id;
   }
@@ -153,9 +174,16 @@ int32_t SPVM__Net__SSLeay__PEM__read_bio_PrivateKey(SPVM_ENV* env, SPVM_VALUE* s
     
     env->die(env, stack, "[OpenSSL Error]read_bio_PrivateKey failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
     
-    int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
-    if (error_id) { return error_id; }
-    error_id = tmp_error_id;
+    if (ERR_GET_REASON(ssl_error) == PEM_R_NO_START_LINE) {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error::PEM_R_NO_START_LINE", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
+    else {
+      int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      error_id = tmp_error_id;
+    }
     
     return error_id;
   }
